﻿using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.AspNetCore.Authentication;
using Microsoft.Extensions.Options;
using System.Security.Claims;
using System.Text.Encodings.Web;
using System.IdentityModel.Tokens.Jwt;
using Ys.Scada.Repository;
using Microsoft.AspNetCore.Server.IIS.Core;
using Microsoft.AspNetCore.Authorization;

namespace Ys.Scada.Backend.Authorization
{

    public class JwtTokoenAuthorizeHandler : JwtBearerHandler
    {
        private readonly IFreeSql _fsql;

        public JwtTokoenAuthorizeHandler(IOptionsMonitor<JwtBearerOptions> options, ILoggerFactory logger, UrlEncoder encoder, ISystemClock clock, IFreeSql freeSql) : base(options, logger, encoder, clock)
        {
            _fsql = freeSql;
        }
        protected override async Task<AuthenticateResult> HandleAuthenticateAsync()
        {
            var endpoint = Context.GetEndpoint();
            if (endpoint == null || endpoint?.Metadata?.GetMetadata<IAllowAnonymous>() != null)
                return AuthenticateResult.NoResult();
            var result = await base.HandleAuthenticateAsync();
            if (result.Succeeded)
            {
                var userId = result.Principal.FindFirstValue(ClaimTypes.NameIdentifier);
                if (string.IsNullOrEmpty(userId))
                    throw new UnauthorizedAccessException();

                var jtiDb = await _fsql.Select<User>().Where(x => x.Id == uint.Parse(userId)).ToOneAsync(x => x.Jti);
                if (int.TryParse(result.Principal.FindFirstValue(JwtRegisteredClaimNames.Jti) ?? "0", out var jti) && jti == jtiDb)
                    return result;
                throw new UnauthorizedAccessException();
            }
            throw new UnauthorizedAccessException();
        }
    }
}
