﻿using AutoMapper;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using System.Security.Claims;

namespace Ys.Scada.Backend.Controllers
{
    [Authorize]
    [Route("api/[controller]")]
    [ApiController]
    public class BaseController : ControllerBase
    {
        protected readonly IFreeSql _fsql;
        protected readonly IMapper _mapper;
        public BaseController(IFreeSql fsql, IMapper mapper)
        {
            _fsql = fsql;
            _mapper = mapper;
        }
        public int UserId
        {
            get
            {
                if (int.TryParse(User.FindFirstValue(ClaimTypes.NameIdentifier), out var userId))
                    return userId;
                throw new UnauthorizedAccessException();
            }
        }
        protected virtual JsonResult OK<T>(T data)
        {
            return new JsonResult(new Result<T> { Code = 200, Data = data });
        }
        protected virtual JsonResult OK()
        {
            return new JsonResult(new Result { Code = 200 });
        }
    }
    public class Result
    {
        public int Code { get; set; }
        public string Msg { get; set; }
    }
    public class Result<T> : Result
    {
        public T Data { get; set; }
    }
}
