﻿using AutoMapper;
using Microsoft.AspNetCore.Mvc;
using Ys.Scada.Backend.Models;
using Ys.Scada.Core;
using Ys.Scada.Repository;

namespace Ys.Scada.Backend.Controllers
{
    public class PermissionController : BaseController
    {
        private readonly ILogger<PermissionController> _logger;
        public PermissionController(ILogger<PermissionController> logger, IFreeSql fsql, IMapper mapper) : base(fsql, mapper)
        {
            _logger = logger;
        }
        [HttpPost("save")]
        public async Task<JsonResult> Save(PermissionModel  permissionModel) 
        {
            Check.NotNull(permissionModel);
            if (permissionModel.Id > 0)
            {
                var model = _mapper.Map<Permission>(permissionModel);
                await _fsql.Update<Permission>().SetSource(model)
                    .IgnoreColumns(o => new{ o.Id })
                    .ExecuteAffrowsAsync();
            }
            else 
            {
                var model = _mapper.Map<Permission>(permissionModel);
                permissionModel.Id = (int)(await _fsql.Insert<Permission>(model).ExecuteIdentityAsync());
            }
            return OK(permissionModel);
        }
        [HttpGet("list")]
        public async Task<JsonResult> List([FromQuery] int page, [FromQuery] int rows, [FromQuery] string? filter) 
        {
            var data = await _fsql.Select<Permission>()
                .WhereIf(!string.IsNullOrEmpty(filter), o => o.Name.Contains(filter) || o.Url.Contains(filter) || o.Code.Contains(filter))
                .OrderByDescending(o => o.CreatedAt)
                .Count(out long count)
                .Page(page, rows)
                .ToListAsync();
            return OK(new PagedData<PermissionModel>(_mapper.Map<List<PermissionModel>>(data), page, rows, (int)count));
        }
        [HttpDelete("{id}")]
        public async Task Delete(int id)
        {
            var permission = await _fsql.Select<Permission>().Where(x => x.Id == id).ToOneAsync();
            Check.NotNull(permission);
            _fsql.Transaction(() =>
            {
                _fsql.Delete<Permission>(id).ExecuteAffrows();
                _fsql.Delete<PermissionUserRoleMapping>().Where(o=>o.PermissionId == id).ExecuteAffrows();
            });
        }
        [HttpGet("menu")]
        public async Task<JsonResult> GetMenus() 
        {
            var data = await _fsql.Select<Permission>().Where(o=>o.MenuType == 0).ToListAsync();
            return OK(data);
        }
    }
}
