﻿using AutoMapper;
using Microsoft.AspNetCore.Mvc;
using Ys.Scada.Backend.Models;
using Ys.Scada.Core;
using Ys.Scada.Repository.Domain;

namespace Ys.Scada.Backend.Controllers
{
    public class ReleaseController : BaseController
    {
        public ReleaseController(IFreeSql fsql, IMapper mapper) : base(fsql, mapper)
        {
        }
        [HttpPost("publish")]
        public async Task<JsonResult> Publish(int docId) 
        {
            var document = await _fsql.Select<Document>().Where(o => o.Id == docId).FirstAsync();
            if (document == null)
            {
                throw new CustomException("数据不存在");
            }
            var rel = _mapper.Map<Release>(document);
            rel.DocId = docId;
            return OK( await _fsql.Insert<Release>(rel).ExecuteIdentityAsync());
        }
        [HttpGet("list")]
        public async Task<JsonResult> List([FromQuery] int page, [FromQuery] int rows, [FromQuery] int docId)
        {
            
            var query = _fsql.Select<Release>().Where(o=>o.DocId == docId);
            var data = await query.OrderByDescending(o => o.CreatedAt)
               .Count(out long count)
               .Page(page, rows)
               .ToListAsync();
            return OK(new PagedData<ReleaseModel>(_mapper.Map<List<ReleaseModel>>(data), page, rows, (int)count));
        }
        [HttpGet("delete")]
        public async Task<JsonResult> Delete(int id)
        {
            var resources = await _fsql.Select<Release>().Where(x => x.Id == id).ToOneAsync();
            Check.NotNull(resources);
            _fsql.Delete<Release>().Where(o => o.Id == id).ExecuteAffrows();
            return OK();
        }
        [HttpGet("get")]
        public async Task<JsonResult> Get(int docId)
        {
            var model = (await _fsql.Select<Release>().Where(o=>o.DocId == docId).OrderByDescending(o=>o.CreatedAt).FirstAsync());
            if (model == null)
                throw new CustomException("数据不存在");
            var viewModel = _mapper.Map<ReleaseModel>(model);
            return OK(viewModel);
        }
    }
}
