﻿using AutoMapper;
using Microsoft.AspNetCore.Mvc;
using System.Text.RegularExpressions;
using Ys.Scada.Backend.Models;
using Ys.Scada.Core;
using Ys.Scada.Core.Common;
using Ys.Scada.Repository;
using Ys.Scada.Repository.Domain;
using static System.Net.Mime.MediaTypeNames;

namespace Ys.Scada.Backend.Controllers
{
    public class ResourcesController : BaseController
    {
        public ResourcesController(IFreeSql fsql, IMapper mapper) : base(fsql, mapper)
        {
        }
        [HttpPost("save")]
        public async Task<JsonResult> Save(ResourcesModel resourcesModel)
        {
            Check.NotNull(resourcesModel);

            if (resourcesModel.Id > 0)
            {
                var oldModel = (await _fsql.Select<Resources>().Where(o => o.Id == resourcesModel.Id).FirstAsync());
                if (oldModel == null)
                {
                    throw new CustomException("数据不存在");
                }
                var model = _mapper.Map<Resources>(resourcesModel);
                await _fsql.Update<Resources>().SetSource(model).IgnoreColumns(o => new { o.Ext, o.UserId }).ExecuteAffrowsAsync();
            }
            else
            {
                Check.NullEmpty(resourcesModel.Name);
                var match = Regex.Match(resourcesModel.Name, "\\.(?'ext'.*)");
                if (!match.Success)
                {
                    throw new CustomException("图片内容错误");
                }
                var ext = match.Groups["ext"].Value;
                resourcesModel.Ext = ext;
                resourcesModel.UserId = UserId;
                var model = _mapper.Map<Resources>(resourcesModel);
                model.CreatedBy = UserId;
                resourcesModel.Id = (int)(await _fsql.Insert<Resources>(model).ExecuteIdentityAsync());
                await Base64ToImage.ConvertBase64ToImage(resourcesModel.Base64, Path.Combine(GlobalSettings.ResourcessPath, $"{resourcesModel.Id}.{ext}"));
            }
            resourcesModel.Base64 = string.Empty;
            return OK(resourcesModel);
        }
        [HttpGet("list")]
        public async Task<JsonResult> List([FromQuery] int page, [FromQuery] int rows, [FromQuery] string? filter, string? type, string? channel)
        {
            if (string.IsNullOrWhiteSpace(channel))
                return OK(new PagedData<ResourcesModel>(new List<ResourcesModel>(), page, rows, 0));
            var query = _fsql.Select<Resources>()
                .WhereIf(!string.IsNullOrEmpty(filter), o => o.Title.Contains(filter) || o.Keywords.Contains(filter));
            query = query.WhereIf(!string.IsNullOrWhiteSpace(type), o => o.Type.Contains(type));
            query = query.Where(o => o.Channel == channel);
            var data = await query
                .OrderByDescending(o => o.CreatedAt)
                .Count(out long count)
                .Page(page, rows)
                .ToListAsync();
            return OK(new PagedData<ResourcesModel>(_mapper.Map<List<ResourcesModel>>(data), page, rows, (int)count));
        }
        [HttpGet("myList")]
        public async Task<JsonResult> MyList([FromQuery] int page, [FromQuery] int rows, [FromQuery] string? filter, string? type, string? channel)
        {
            var query = _fsql.Select<Resources>()
                .WhereIf(!string.IsNullOrEmpty(filter), o => o.Title.Contains(filter) || o.Keywords.Contains(filter));
            query = query.WhereIf(!string.IsNullOrWhiteSpace(type), o => o.Type.Contains(type));
            query = query.Where(o => o.Channel == channel);
            query = query.Where(o => o.UserId == UserId);
            var data = await query.OrderByDescending(o => o.CreatedAt)
               .Count(out long count)
               .Page(page, rows)
               .ToListAsync();
            return OK(new PagedData<ResourcesModel>(_mapper.Map<List<ResourcesModel>>(data), page, rows, (int)count));
        }
        [HttpGet("delete")]
        public async Task<JsonResult> Delete(int id)
        {
            var resources = await _fsql.Select<Resources>().Where(x => x.Id == id).ToOneAsync();
            Check.NotNull(resources);
            _fsql.Delete<Resources>().Where(o => o.Id == id).ExecuteAffrows();
            return OK();
        }
        [HttpGet("get")]
        public async Task<JsonResult> Get(int id)
        {
            var model = (await _fsql.Select<Resources>().Where(o => o.Id == id).FirstAsync());
            if (model == null)
                throw new CustomException("数据不存在");
            var viewModel = _mapper.Map<ResourcesModel>(model);
            return OK(viewModel);
        }
        [HttpPost("release")]
        public async Task<JsonResult> Release(ResourcesReleaseModel resourcesModel)
        {
            var oldModel = (await _fsql.Select<Resources>().Where(o => o.Id == resourcesModel.Id).FirstAsync());
            if (oldModel == null)
                throw new CustomException("数据不存在");
            var model = _mapper.Map<Resources>(resourcesModel);
            await _fsql.Update<Resources>().Set(o => o.Channel, model.Channel).Set(o => o.Description, model.Description).Set(o => o.Title, model.Title).Set(o => o.Keywords, model.Keywords).Where(o=>o.Id == resourcesModel.Id).ExecuteAffrowsAsync();
            return OK();
        }
    }
}
