﻿using AutoMapper;
using Microsoft.AspNetCore.Mvc;
using Ys.Scada.Backend.Models;
using Ys.Scada.Core;
using Ys.Scada.Repository;

namespace Ys.Scada.Backend.Controllers
{
    public class RoleController : BaseController
    {
        public RoleController(ILogger<RoleController> logger, IFreeSql fsql, IMapper mapper) : base(fsql, mapper)
        {
        }
        [HttpPost("save")]
        public async Task<JsonResult> Save(RoleModel roleModel)
        {
            Check.NotNull(roleModel);
            if (roleModel.Id > 0)
            {
                var include = await _fsql.Select<UserRole>().ToOneAsync(o => o.Id == roleModel.Id);
                if (!include)
                    throw new CustomException("角色不存在");
                var role = _mapper.Map<UserRole>(roleModel);
                _fsql.Update<UserRole>().SetSource(role).ExecuteAffrows();

                var rolePermMapping = _fsql.Select<PermissionUserRoleMapping>().Where(o => o.RoleId == roleModel.Id).ToList();
                if (rolePermMapping.Any())
                {
                    rolePermMapping.ForEach(o =>
                    {
                        _fsql.Delete<PermissionUserRoleMapping>(o.Id).ExecuteAffrows();
                    });
                }
                if (roleModel.Permissions != null && roleModel.Permissions.Any())
                {
                    roleModel.Permissions.ForEach(o =>
                    {
                        //var model = _mapper.Map<PermissionUserRoleMapping>(o);
                        _fsql.Insert(new PermissionUserRoleMapping { PermissionId = o.Id, RoleId = roleModel.Id }).ExecuteAffrows();
                    });
                }
            }
            else
            {
                var include = await _fsql.Select<UserRole>().ToOneAsync(o => o.Name == roleModel.Name);
                if (include)
                    throw new CustomException("角色名称重复");

                _fsql.Transaction(() =>
                {
                    var role = _mapper.Map<UserRole>(roleModel);
                    roleModel.Id = (int)_fsql.Insert<UserRole>(role).ExecuteIdentity();

                    var perms = roleModel.Permissions;
                    if (perms != null)
                    {
                        perms.ForEach(o =>
                        {
                            _fsql.Insert<PermissionUserRoleMapping>(new PermissionUserRoleMapping
                            {
                                PermissionId = o.Id,
                                RoleId = roleModel.Id
                            }).ExecuteAffrows();
                        });
                    }
                });
            }
            return OK(roleModel);
        }
        [HttpGet("list")]
        public async Task<JsonResult> List([FromQuery] int page, [FromQuery] int rows, [FromQuery] string? filter)
        {
            var data = await _fsql.Select<UserRole>()
                .WhereIf(!string.IsNullOrEmpty(filter), o => o.Name.Contains(filter))
                .OrderByDescending(o => o.CreatedAt)
                .Count(out long count)
                .Page(page, rows)
                .ToListAsync();
            return OK(new PagedData<RoleModel>(_mapper.Map<List<RoleModel>>(data), page, rows, (int)count));
        }
        [HttpDelete("{id}")]
        public async Task Delete(int id)
        {
            var role = await _fsql.Select<UserRole>().Where(x => x.Id == id).ToOneAsync();
            Check.NotNull(role);
            _fsql.Transaction(() =>
            {
                _fsql.Delete<UserRole>(id).ExecuteAffrows();
                _fsql.Delete<UserRoleMapping>().Where(o => o.UserRoleId == id).ExecuteAffrows();
            });
        }
    }
}
