﻿using Newtonsoft.Json.Serialization;
using Newtonsoft.Json;
using System.Net;
using Ys.Scada.Core;
using Ys.Scada.Backend.Controllers;

namespace Ys.Scada.Backend
{
    public class ErrorHandlingMiddleware
    {
        private static readonly JsonSerializerSettings setting = new()
        {
            ContractResolver = new CamelCasePropertyNamesContractResolver(),
            NullValueHandling = NullValueHandling.Ignore
        };

        private readonly RequestDelegate next;

        private readonly ILogger<ErrorHandlingMiddleware> _logger;

        public ErrorHandlingMiddleware(RequestDelegate next, ILogger<ErrorHandlingMiddleware> logger)
        {
            this.next = next;
            _logger = logger;
        }

        public async Task Invoke(HttpContext context /* other dependencies */)
        {
            try
            {
                await next(context);
            }
            catch (Exception ex)
            {
                await HandleExceptionAsync(context, ex);
            }
        }

        private Task HandleExceptionAsync(HttpContext context, Exception exception)
        {
            var response = new Result<string>();
            switch (exception)
            {
                case UnauthorizedAccessException:
                case { InnerException: UnauthorizedAccessException }:
                    {
                        context.Response.StatusCode = (int)HttpStatusCode.Unauthorized;
                        response.Code = 600;
                        break;
                    }
                case CustomException ex:
                    {
                        context.Response.StatusCode = (int)HttpStatusCode.BadRequest;
                        response.Code = 500;
                        response.Msg = ex.Message;
                        break;
                    }
                default:
                    {
                        context.Response.StatusCode = (int)HttpStatusCode.InternalServerError;
                        response.Msg = "服务器异常";
                        _logger.LogError(exception, exception.Message);
                        break;
                    }
            }
            var result = JsonConvert.SerializeObject(response, setting);
            context.Response.ContentType = "application/json";
            return context.Response.WriteAsync(result);
        }
    }
}
