﻿using Ys.Scada.Core.EventBus.Consumer;
using Ys.Scada.Core.EventModels;
using Ys.Scada.Manager;
using Ys.Scada.RabbitMQ;
using Newtonsoft.Json;
using Ys.Scada.Backend.WS;
using Ys.Scada.Core.MQ;
using System.Text;
using SixLabors.Fonts.Tables.AdvancedTypographic;
using Ys.Scada.Core.Common;

namespace Ys.Scada.Backend.EventConsumers
{
    public class TagDataConsumerData : IConsumer<TransportMessage>
    {
        private readonly DataServer _dataServer;
        private readonly DedicatedThreadTaskScheduler _scheduler;
        public TagDataConsumerData(DataServer dataServer)
        {
            _dataServer = dataServer;
            _scheduler = new DedicatedThreadTaskScheduler(10);
        }
        public Task HandleEvent(TransportMessage eventMessage)
        {
            var message = Encoding.UTF8.GetString(eventMessage.Body.ToArray());
            if (!string.IsNullOrWhiteSpace(message))
            {
                var tags = JsonConvert.DeserializeObject<List<TagDataEventModel>>(message);
                if (tags.Any())
                {
                    Parallel.ForEach(tags, new ParallelOptions
                    {
                        MaxDegreeOfParallelism = 10,
                        TaskScheduler = _scheduler
                    },
                    (tag) =>
                    {
                        var key = Global.GetVariableKey(tag.Id);
                        ForgeCache.Instance.SetValue(key, tag.VL);
                        _dataServer.MyMulticast(tag).GetAwaiter().GetResult();
                    });
                }
            }
            return Task.CompletedTask;
        }
    }
}
