﻿using System.ComponentModel.DataAnnotations;
using Ys.Scada.Core;

namespace Ys.Scada.Backend.Models
{
    public class DataSourceModel
    {
        /// <summary>
        /// 数据源自增id
        /// </summary>
        public int Id { get; set; } = 0;
        /// <summary>
        /// 项目id，非页面id，后端会校验，如果是非项目id会直接返回错误
        /// </summary>
        [Required(ErrorMessage = "项目编号必填")]
        public int DocId { get; set; }
        /// <summary>
        /// 数据源类型，可传参数为rabbitmq、kafka、mqtt。
        /// 前期我们主要用rabbitmq
        /// </summary>
        [Required(ErrorMessage = "数据源类型必填")]
        public string Type { get; set; }
        /// <summary>
        /// 数据源别名
        /// </summary>
        [Required(ErrorMessage = "数据源名称必填")]
        public string Name { get; set; }
        /// <summary>
        /// 数据源地址
        /// </summary>
        [Required(ErrorMessage = "数据源地址必填")]
        public string Host { get; set; }
        /// <summary>
        /// 数据源端口
        /// </summary>
        [Required(ErrorMessage = "数据源端口必填")]
        public int Port { get; set; }
        public string? Ext { get; set; }
        /// <summary>
        /// 通道连接配置，json格式，以rabbitmq为例
        /// {"exchangeType":"topic","hostName":"192.168.2.221","password":"ys123456","userName":"ys","exchangeName":"exchang","queueName":"cnm","topic":"cnm","port":5672,"queueArguments":null,"connectionFactoryOptions":null,"basicQosOptions":null}
        /// 
        /// </summary>
        [Required(ErrorMessage = "数据源配置必填")]
        public string? Data { get; set; }
        /// <summary>
        /// 读或者写
        /// 读表示face从数据源里面取实时数据
        /// 写表示face把数据写入到数据源
        /// </summary>
        [Required(ErrorMessage = "数据源读写类型必填")]
        public string RWType { get; set; }
        public DateTime CreatedAt { get; set; }
        [Required(ErrorMessage = "配置方式必填")]
        public string ConfigType { get; set; }
        public string? Router { get; set; }
        public int Status { get; set; }
    }
}
