﻿using SixLabors.ImageSharp.PixelFormats;
using SixLabors.ImageSharp;
using SixLabors.Fonts;
using SixLabors.ImageSharp.Processing;
using SixLabors.ImageSharp.Drawing.Processing;

namespace Ys.Scada.Backend
{
    public interface IValidateCodeOperate
    {
        /// <summary>
        /// 生成验证码
        /// </summary>
        /// <param name="CodeLength">验证码长度</param>
        /// <param name="Width">宽度</param>
        /// <param name="Height">高度</param>
        /// <param name="FontSize">字体大小</param>
        /// <returns></returns>
        Task<(string code, string)> CreateValidate(int CodeLength, int Width, int Height, int FontSize);
    }
    public class ValidateCodeOperate : IValidateCodeOperate
    {
        private static readonly Color[] Colors = { Color.Black, Color.Red, Color.Blue, Color.Green, Color.Orange, Color.Brown, Color.Brown, Color.DarkBlue };
        private static readonly char[] Chars = { '2', '3', '4', '5', '6', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'J', 'K', 'L', 'M', 'N', 'P', 'R', 'S', 'T', 'W', 'X', 'Y' };
        /// <summary>
        /// 生成验证码
        /// </summary>
        /// <param name="CodeLength">验证码长度</param>
        /// <param name="Width">宽度</param>
        /// <param name="Height">高度</param>
        /// <param name="FontSize">字体大小</param>
        /// <returns></returns>
        public async Task<(string code, string)> CreateValidate(int CodeLength, int Width, int Height, int FontSize)
        {
            var code = await GenCodeAsync(CodeLength);
            var r = new Random();
            using var image = new Image<Rgba32>(Width, Height);
            // 字体
            var font = SystemFonts.CreateFont(SystemFonts.Families.First().Name,
                FontSize, FontStyle.Bold);
            image.Mutate(ctx =>
            {
                // 白底背景
                ctx.Fill(Color.White);
                // 画验证码
                for (int i = 0; i < code.Length; i++)
                {
                    ctx.DrawText(code[i].ToString(), font, Colors[r.Next(Colors.Length)], new PointF(20 * i + 5, r.Next(1, 10)));
                }
                // 画干扰线
                for (int i = 0; i < 6; i++)
                {
                    var pen = new SolidPen(Colors[r.Next(Colors.Length)], 1);
                    var p1 = new PointF(r.Next(Width), r.Next(Height));
                    var p2 = new PointF(r.Next(Width), r.Next(Height));
                    ctx.DrawLine(pen, p1, p2);
                }
                // 画噪点
                for (int i = 0; i < 80; i++)
                {
                    var pen = new SolidPen(Colors[r.Next(Colors.Length)], 1);
                    var p1 = new PointF(r.Next(Width), r.Next(Height));
                    var p2 = new PointF(p1.X + 1f, p1.Y + 1f);
                    ctx.DrawLine(pen, p1, p2);
                }
            });
            using var ms = new MemoryStream();
            image.SaveAsPng(ms);
            return (code, Convert.ToBase64String(ms.ToArray()));
        }
        /// <summary>
        /// 获取验证码
        /// </summary>
        /// <param name="num">验证码长度</param>
        /// <returns></returns>
        private async Task<string> GenCodeAsync(int num)
        {
            return await Task.Run(() =>
            {
                var code = string.Empty;
                var r = new Random();
                for (int i = 0; i < num; i++)
                {
                    code += Chars[r.Next(Chars.Length)].ToString();
                }
                return code;
            });
        }
    }
}
