﻿using static Ys.Scada.Backend.Controllers.RealTimeDataController;
using Ys.Scada.Core;
using Ys.Scada.Core.EventModels;
using static Confluent.Kafka.ConfigPropertyNames;
using Ys.Scada.Core.EventBus.Publisher;
using System.Threading.Channels;
using Ys.Scada.Core.Common;
using Ys.Scada.Core.MQ;

namespace Ys.Scada.Backend.WS
{
    public class TagDataConsumer
    {
        private readonly ChannelReader<TransportMessage> _reader;
        private readonly IEventPublisher _eventPublisher;
        public TagDataConsumer(ChannelReader<TransportMessage> reader, IEventPublisher eventPublisher)
        {
            _reader = reader;
            _eventPublisher = eventPublisher;
        }
        public async Task ConsumeDataAsync(CancellationToken cancellationToken = default)
        {
            while (await _reader.WaitToReadAsync(cancellationToken))
            {
                if (_reader.TryRead(out var message))
                {
                  await  _eventPublisher.Publish(message);
                }
            }
        }
    }
    public class TagDataChannelConsumer 
    {
        private int handlerCount;
        private readonly MemoryChannel<TransportMessage> _channel;
        private readonly IEventPublisher _eventPublisher;
        private DedicatedThreadTaskScheduler scheduler;
        public TagDataChannelConsumer(MemoryChannel<TransportMessage> channel, IEventPublisher eventPublisher, string count)
        {
            _channel = channel;
            handlerCount = int.TryParse(count, out int c)  ? c : 50;
            _eventPublisher = eventPublisher;
            scheduler = new DedicatedThreadTaskScheduler(handlerCount);
        }
        public Task Excute()
        {
            Enumerable.Range(1, handlerCount).ToList().ForEach(_ =>
            {
                Task.Factory.StartNew(async () =>
                {
                    var consumer = new TagDataConsumer(_channel.Channel.Reader, _eventPublisher);
                    await consumer.ConsumeDataAsync(CancellationToken.None);
                }, CancellationToken.None, TaskCreationOptions.LongRunning, scheduler);
            });
            return Task.CompletedTask;
        }
    }
    public static class RunTagDataConsumer 
    {
        public static void RunConsumer(this IApplicationBuilder applicationBuilder, IConfiguration configuration)
        {
            var services = applicationBuilder.ApplicationServices;
            var channel = services.GetRequiredService<MemoryChannel<TransportMessage>>();
            var publisher=  services.GetRequiredService<IEventPublisher>();
            var count = configuration["TagDataConsumer:Count"];
            var consumer = new TagDataChannelConsumer(channel, publisher, count);
            consumer.Excute();
        }
    }
}

