﻿using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ys.Scada.Core.Common
{
    public sealed class DedicatedThreadTaskScheduler : TaskScheduler
    {
        private readonly BlockingCollection<Task> _tasks = new();
        private readonly List<Thread> _threads;
        protected override IEnumerable<Task>? GetScheduledTasks() => _tasks;
        protected override void QueueTask(Task task) => _tasks.Add(task);
        protected override bool TryExecuteTaskInline(Task task, bool taskWasPreviouslyQueued) => false;
        public DedicatedThreadTaskScheduler(int threadCount)
        {
            _threads = new List<Thread>();
            for (int index = 0; index < threadCount; index++)
            {
                _threads.Add(new Thread(_ =>
                {
                    while (true) { TryExecuteTask(_tasks.Take()); }
                }));
            }
            Array.ForEach(_threads.ToArray(), it => it.Start());
        }
    }
}
