﻿using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ys.Scada.Core.Common
{
    public class ForgeCache
    {
        private readonly ConcurrentDictionary<string, object> _cache;
        private static ForgeCache _forge;
        private readonly static object _obj = new object();
        private ForgeCache()
        {
            _cache = new ConcurrentDictionary<string, object>();
        }
        /// <summary>
        /// 实例
        /// </summary>
        public static ForgeCache Instance
        {
            get
            {
                if (_forge == null)
                {
                    lock (_obj)
                    {
                        if (_forge == null)
                        {
                            _forge = new ForgeCache();
                        }
                    }
                }
                return _forge;
            }
        }
        /// <summary>
        /// 获取值
        /// </summary>
        /// <param name="key">Key</param>
        /// <returns></returns>
        public T GetValue<T>(string key)
        {
            if (_cache.TryGetValue(key, out object value))
                return (T)value;
            return default;
        }
        /// <summary>
        /// 设置值
        /// </summary>
        /// <param name="key">Key</param>
        /// <param name="value">Value</param>
        public void SetValue<T>(string key, T value)
        {
            _cache.AddOrUpdate(key, value, (n, t) => value);
        }
        /// <summary>
        /// 删除值
        /// </summary>
        /// <param name="key">Key</param>
        public void DeleteValue(string key)
        {
            _cache.Remove(key, out object value);
        }
    }
}
