﻿using Microsoft.Extensions.Logging;
using Ys.Scada.Core.EventBus.Consumer;
using Ys.Scada.Core.EventBus.Subscriber;

namespace Ys.Scada.Core.EventBus.Publisher
{
    /// <summary>
    /// 时间发布
    /// </summary>
    public class EventPublisher : IEventPublisher
    {
        private readonly ISubscriber subscriptionService;
        private readonly ILogger logger;
        public EventPublisher(ISubscriber subscriptionService, ILogger<EventPublisher> logger)
        {
            this.subscriptionService = subscriptionService;
            this.logger = logger;
        }
        /// <summary>
        /// 发布
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="consumer">消费者</param>
        /// <param name="eventMessage">发送的事件消息</param>
        /// <returns></returns>
        protected virtual Task Publish<T>(IConsumer<T> consumer, T eventMessage)
        {
            return consumer.HandleEvent(eventMessage);
        }
        /// <summary>
        /// 发布
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="eventMessage">发送的事件消息</param>
        /// <returns></returns>
        public virtual Task Publish<T>(T eventMessage)
        {
            var subscriptions = subscriptionService.GetSubscriptions<T>();
            subscriptions.ToList().ForEach(x => Publish(x, eventMessage));
            return Task.CompletedTask;
        }
    }
}
