﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Ys.Scada.Core;

namespace Microsoft.Extensions.DependencyInjection
{
    public static class FreeSqlExtensions
    {
        public static IServiceCollection AddFreesql(this IServiceCollection services, DbSettings dbSettings)
        {
            var type = FreeSql.DataType.MySql;
            if (!string.IsNullOrWhiteSpace(dbSettings.DbType))
            {
                switch (dbSettings.DbType.ToLower())
                {
                    case "sqlite":
                        type = FreeSql.DataType.Sqlite;
                        break;
                    case "sqlserver":
                        type = FreeSql.DataType.SqlServer;
                        break;
                }
            }
            IFreeSql fsql = new FreeSql.FreeSqlBuilder()
            .UseConnectionString(type, dbSettings.ConnString)
            .UseAutoSyncStructure(true)
            //.UseMonitorCommand(cmd => Console.Write(cmd.CommandText))
            .Build();
            fsql.Aop.ConfigEntityProperty += (s, e) =>
            {
                if (e.Property.Name == "CreatedAt")
                {
                    e.ModifyResult.CanInsert = true;
                    e.ModifyResult.CanUpdate = false;
                }
            };
            services.AddSingleton(fsql);
            return services;
        }
    }
}
